"""Module for reading gshhs binary coastline formats"""

import struct, os
import numarray as NA
from matplotlib.mlab import conv

class gshhs:
    def _get_header(self):
        nb_i = struct.calcsize('i') # 4 bytes in 'int'
        nb_h = struct.calcsize('h') # 2 bytes in 'short int'
        hdr_len = nb_i*8 + nb_h*2
        bhdr = self.f.read(hdr_len)
        if not bhdr:
            return None
        hdr = {}; n=0
        # integer header information (big-endien)
        hdr['id']     = struct.unpack('>i',bhdr[n:n+nb_i])[0];        n += nb_i
        hdr['n']      = struct.unpack('>i',bhdr[n:n+nb_i])[0];        n += nb_i
        hdr['level']  = struct.unpack('>i',bhdr[n:n+nb_i])[0];        n += nb_i
        hdr['west']   = struct.unpack('>i',bhdr[n:n+nb_i])[0]*1.0e-6; n += nb_i
        hdr['east']   = struct.unpack('>i',bhdr[n:n+nb_i])[0]*1.0e-6; n += nb_i
        hdr['south']  = struct.unpack('>i',bhdr[n:n+nb_i])[0]*1.0e-6; n += nb_i
        hdr['north']  = struct.unpack('>i',bhdr[n:n+nb_i])[0]*1.0e-6; n += nb_i
        hdr['area']   = struct.unpack('>i',bhdr[n:n+nb_i])[0];        n += nb_i
        # short integer header information
        hdr['grenwich']  = struct.unpack('>h',bhdr[n:n+nb_h])[0];     n += nb_h
        hdr['source']    = struct.unpack('>h',bhdr[n:n+nb_h])[0];
        return hdr
    
    def _get_poly(self,n):
        n_i4 = struct.calcsize('i')
        bdata = self.f.read(2*n_i4*n)
	# unpack big-endian micro degrees
        data  = struct.unpack('>'+str(2*n)+'i',bdata)[:]
        [self.coastlons.append(d*1.e-6) for d in data[::2]]
        [self.coastlats.append(d*1.e-6) for d in data[1::2]]
           
    def __init__(self,resolution='i',data_dir=None,area_thresh=0.):
        if data_dir is None:
            data_dir = os.environ.get('GSHHS_PATH')
        self.f = open(os.path.join(data_dir,'gshhs_' + resolution + '.b'),'rb')
        self.coastlons=[]; self.coastlats=[];
        self.coastsegind=[]; self.coastsegtype=[]; self.coastsegarea=[]
        hdr = gshhs._get_header(self)
        while hdr:
            if hdr['area'] > area_thresh:
                self._get_poly(hdr['n'])
                self.coastsegind.append(len(self.coastlons))
                self.coastsegtype.append(hdr['level'])
                self.coastsegarea.append(hdr['area'])
            else:
                self.f.seek(2*4*hdr['n'],1)
            hdr = gshhs._get_header(self)
	self.f.close()
	    
if __name__ == '__main__':
    import time, sys
    resolution = sys.argv[1]
    area_thresh = float(sys.argv[2])
    t1 = time.clock()
    m = gshhs(resolution=resolution,area_thresh=area_thresh)
    t2 = time.clock()
    print 'binary read time: ' + str(t2-t1) + ' seconds'
    print len(m.coastlons)
    t1 = time.clock()
    # read in coastline data.
    coastlons = []; coastlats = []; coastsegind = []; coastsegarea = []; coastsegtype = []
    for line in open(os.path.join('/sw/share/basemap','gshhs_'+resolution+'.txt')):
        linesplit = line.split()
        if line.startswith('P'):
            area = float(linesplit[5])
            if area > area_thresh:
                coastsegind.append(len(coastlons))
                coastsegtype.append(int(linesplit[3]))
                coastsegarea.append(float(linesplit[5]))
            continue
        # lon/lat
        if area > area_thresh:
            lon, lat = [float(val) for val in linesplit]
            coastlons.append(lon)
            coastlats.append(lat)
    t2 = time.clock()
    print 'ascii read time: ' + str(t2-t1) + ' seconds'
    print len(coastlons)
